class TokenBox(object):
    def __init__(self, url, username, password, margin = 10, request_now = False):
        self.URL = url
        self.Username = username
        self.Password = password
        self.Token = None
        self.Expiration = 0
        self.Encoded = None
        self.Margin = margin
        if request_now:
            self.renewIfNeeded()
        
    def renewIfNeeded(self):
        need_to_renew = self.Token is None or time.time() > self.Expiration - self.Margin
        if need_to_renew:
            from .rfc2617 import digest_client
            status, body = digest_client(self.URL, self.Username, self.Password)
            if status/100 == 2:
                encoded = body.strip()
                t = SignedToken.decode(encoded)
                self.Token = t
                self.Encoded = encoded
                self.Expiration = t.expiration
            else:
                raise SignedTokenAuthoriztionError(body)
    
    @property
    def token(self):
        self.renewIfNeeded()
        return self.Encoded
            
        



import sys, getopt, os, json, pprint, time
from metacat.webapi import MetaCatClient, MCWebAPIError, MCInvalidMetadataError
from metacat.ui.cli import CLI, CLICommand, InvalidOptions, InvalidArguments

def read_file_list(opts):
    if "-i" in opts or "--ids" in opts:
        field = "fid"
        source = opts.get("-i") or opts.get("--ids")
    elif "-n" in opts or "--names" in opts:
        field = "name"
        source = opts.get("-n") or opts.get("--names")
    else:
        raise InvalidArguments("File list must be specified either with --names or --ids")
        
    if source == "-":
        lst = (x.strip() for x in sys.stdin.readlines())
    elif source.startswith("@"):
        lst = (x.strip() for x in open(source[1:], "r").readlines())
    else:
        lst = source.split(",")

    return [{field:x} for x in lst if x]

def parse_namespace_name(spec, default_namespace=None):
    if ":" in spec:
        return tuple(spec.split(":", 1))
    else:
        return default_namespace, spec

class DeclareCommand(CLICommand):
    
    Opts = ("N:j:p:m:s:", ["json=", "namespace=", "sample", "parents=", "metadata=", "size="])
    Usage = """
    signle file:
        declare [options] [<file namespace>:]<filename> [<dataset namespace>:]<dataset>
            -s|--size <file size, bytes>
            -N|--namespace <default namespace>
            -p|--parents <parent_id>,... 
            -m|--metadata <JSON metadata file>  - if unspecified, file will be declared with empty metadata

    multiple files:
        declare [-N|--namespace <default namespace>] -j|--json <json file> [<dataset namespace>:]<dataset>

        declare --sample                                   - print JSON declaration file sample
    """
    
    DeclareSample = json.dumps([
        {        
            "namespace":"test",
            "name":"file1.dat",
            "metadata": {
                "pi": 3.14,
                "version":"1.0",
                "format":"raw",
                "done":True
            },
            "size": 1234,
            "parents":[ "4722545", "43954" ]
        },
        {        
            "namespace":"test",
            "name":"file2.dat",
            "metadata": {
                "e": 2.718,
                "version":"1.0",
                "format":"raw",
                "done":False
            },
            "size": 1734,
            "parents":[ "4723345", "4322954" ]
        }
    ], indent=4, sort_keys=True)

    def __call__(self, command, client, opts, args):
        default_namespace = opts.get("-N") or opts.get("--namespace")

        if "--sample" in opts:
            print(self.DeclareSample)
            return
    
        size = opts.get("-s", opts.get("--size"))

        if "-j" in opts or "--json" in opts:
            json_file = opts.get("-j") or opts.get("--json")
            files = json.load(open(json_file, "r"))       # parse to validate JSON
            dataset_namespace, dataset_name = parse_namespace_name(args[0], default_namespace)
            if dataset_namespace is None:
                raise InvalidArguments("dataset not specified")
                sys.exit(1)
        else:
            parents = opts.get("-p") or opts.get("--parents")
            if parents:
                parents = parents.split(",")
            file_spec, dataset_spec = args
            file_namespace, file_name = parse_namespace_name(file_spec, default_namespace)
            if not file_namespace:
                raise InvalidArguments("File namespace not specified")
                sys.exit(1)
            dataset_namespace, dataset_name = parse_namespace_name(dataset_spec, default_namespace)
            if not dataset_namespace:
                raise InvalidArguments("Dataset namespace not specified")
                sys.exit(1)

            metadata_file = opts.get("-m") or opts.get("--metadata")
            if metadata_file:
                metadata = json.load(open(metadata_file, "r"))
            else:
                metadata = {}
            assert isinstance(metadata, dict)
            file_data = {
                    "namespace":    file_namespace,
                    "name":         file_name,
                    "metadata":     metadata
                }
            if size is not None:
                file_data["size"] = size
            if parents:
                file_data["parents"] = parents
            files = [file_data]
    
        try:
            response = client.declare_files(f"{dataset_namespace}:{dataset_name}", files, namespace = default_namespace)    
            print(response)
        except MCInvalidMetadataError as e:
            print(e)
            sys.exit(1)


class ShowCommand(CLICommand):

    Opts = ("jmpi:l:In", ["json","meta-only","pretty","names-only","lineage","provenance","ids"])
    Usage = """[<options>] (-i <file id>|<namespace>:<name>)
            -m|--meta-only            - metadata only
            -j|--json                 - as JSON
            -p|--pretty               - pretty-print information
            -l|--lineage|--provenance (p|c)        - parents or children instead of the file itself
            -n|--name-only            - print namespace:name only
            -I|--ids-only             - for parents and children, print file IDs only
    """

    def __call__(self, command, client, opts, args):
        if not args and "-i" not in opts:
            raise InvalidArguments("Either -i <file id> or <namespace:name> must be specified")
        elif not args and "-i" in opts:
            raise InvalidArguments("Either -i <file id> or <namespace:name> must be specified")
            

        #print("opts:", opts,"    args:", args)
    
        as_json = "--json" in opts or "-j" in opts
        pretty = "-p" in opts or "--pretty" in opts
        provenance = opts.get("-l") or opts.get("--lineage") or opts.get("--provenance")
        names_only = "--names-only" in opts or "-n" in opts
        ids_only = "--ids-only" in opts or "-I" in opts
        meta_only = "--meta-only" in opts or "-m" in opts
    
        did = fid = None
    
        if args:
            did = args[0]
        else:
            fid = opts["-i"]

        data = client.get_file(did=did, fid=fid, with_provenance=True)
        if provenance:
            ids = data["parents"] if provenance == "p" else data["children"]
            if ids:
                lst = [dict(fid=fid) for fid in ids]
                related = client.get_files(lst)
                if as_json:
                    print(json.dumps(related))
                elif pretty:
                    pprint.pprint(related)
                else:
                    for f in related:
                        if ids_only:
                            print(f["fid"])
                        else:
                            print("%(namespace)s:%(name)s" % f)
        else:
            if meta_only:
                data = data.get("metadata", {})
            if pretty:
                pprint.pprint(data)
            elif as_json:
                print(json.dumps(data))
            else:
                for k, v in sorted(data.items()):
                    if k != "metadata":
                        print("%-15s:\t%s" % (k, v))
                if "metadata" in data:
                    print("%-15s:\t" % ("metadata",), end="")
                    pprint.pprint(data["metadata"])
                    
class UpdateCommand(CLICommand):
    
    Opts = ("i:n:N:r", ["namespace=", "names=", "ids=", "sample", "replace"])
    Usage = """[<options>] (@<JSON file with metadata>|'<JSON expression>')

            -r|--replace          - replace metadata, otherwise update

            list files by name
            -N|--namespace <default namespace>           - default namespace for files
            -n|--names <file namespace>:<file name>[,...]
            -n|--names -          - read the list from stdin
            -n|--names @<file>    - read the list from file

            list files by file id
            -i|--ids <file id>[,...] 
            -i|--ids -            - read the list from stdin
            -i|--ids @<file>      - read the list from file
    """

    UpdateSample = json.dumps([
            {        
                "name":"test:file1.dat",
                "metadata": {
                    "pi": 3.14,
                    "version":"1.0",
                    "format":"raw",
                    "done":True
                },
                "parents":[ "4722545", "43954" ]
            },
            {        
                "name":"test:file1.dat",
                "metadata": {
                    "pi": 3.14,
                    "version":"1.0",
                    "format":"raw",
                    "done":True
                },
                "parents":[ "4722545", "43954" ]
            },
            {        
                "fid":"54634",
                "metadata": {
                    "q": 2.8718,
                    "version":"1.1",
                    "format":"processed",
                    "params": [1,2],
                    "done":False
                }
            }
        ],
        indent=4, sort_keys=True)
        
    def __call__(self, command, client, opts, args):
        
        if "--sample" in opts:
            print(self.UpdateSample)
            sys.exit(0)
        
        mode = "replace" if ("-r" in opts or "--replace" in opts) else "update"
        namespace = opts.get("-N") or opts.get("--namespace")
    
        file_list = read_file_list(opts)
        names = fids = None
        if "-i" in opts or "--ids" in opts:
            fids = [f["fid"] for f in file_list]
        else:
            names = [f["name"] for f in file_list]
        
        meta = args[0]
        if meta.startswith('@'):
            meta = json.load(open(meta[1:], "r"))
        elif meta == "-":
            meta = json.load(sys.stdin)
        else:
            meta = json.loads(meta)

        try:    response = client.update_file_meta(meta, names=names, fids=fids, mode=mode, namespace=namespace)
        except MCInvalidMetadataError as e:
            data = e.json()
            print(data["message"], file=sys.stderr)
            for error in data.get("metadata_errors", []):
                print("  {name} = {value}: {reason}".format(**error), file=sys.stderr)
            sys.exit(1)
        except Exception as e:
            print(e)
            sys.exit(1)
        print(response)

class AddCommand(CLICommand):
    
    Opts = ("i:j:n:N:", ["namespace=", "json=", "names=", "ids=", "sample"])
    Usage = """[<options>] <dataset namespace>:<dataset name>

            list files by name
            -N|--namespace <default namespace>           - default namespace for files
            -n|--names <file namespace>:<file name>[,...]
            -n|--names -          - read the list from stdin
            -n|--names @<file>    - read the list from file
    
            list files by file id
            -i|--ids <file id>[,...] 
            -i|--ids -            - read the list from stdin
            -i|--ids @<file>      - read the list from file

            read file list from JSON file
            -j|--json <json file>
    """
    MinArgs = 1
    
    AddSample = json.dumps(
        [
            {        
                "name":"test:file1.dat"
            },
            {        
                "name":"test:file1.dat"
            },
            {        
                "fid":"54634"
            }
        ],
        indent=4, sort_keys=True
    )

    def __call__(self, command, context, opts, args):
        if "--sample" in opts:
            print(json.dumps(_add_smaple, sort_keys=True, indent=4, separators=(',', ': ')))
            sys.exit(0)

        file_list = []

        if "-j" in opts or "--json" in opts:
            file_list = json.load(open(opts.get("-f") or opts.get("--files"), "r"))
        else:
            file_list = read_file_list(opts)

        dataset = args[-1]
        namespace = opts.get("-N") or opts.get("--namespace")
        out = client.add_files(dataset, file_list, namespace=namespace)
        print(out)

FileCLI = CLI(
    "declare",  DeclareCommand(),
    "add",      AddCommand(),
    "update",   UpdateCommand(),
    "show",     ShowCommand()
)


import time, uuid, sys
from .py3 import to_bytes, to_str
from datetime import timedelta
import jwt

SignedTokenExpiredError = jwt.ExpiredSignatureError
SignedTokenImmatureError = jwt.ImmatureSignatureError
SignedTokenUnacceptedAlgorithmError = jwt.InvalidAlgorithmError
SignedTokenSignatureVerificationError = jwt.InvalidSignatureError

class SignedToken(object):
    
    def __init__(self, payload={}, subject=None, expiration=None, issuer=None, **claims):
        self.Encoded = None
        
        now = time.time()
        
        self.Payload = {
            "iat":  now,
            "nbf":  now,
            "sub":  subject,
            "exp":  expiration if (expiration is None or expiration > 10*365*24*3600) else now + expiration,
            "jti":  uuid.uuid4().hex[-8:]
        }
        if issuer:
            self.Payload["iss"] = issuer
        self.Payload.update(claims)
        self.Payload.update(payload)
        
    def __str__(self):
        data = {"ctime":time.ctime(self.Payload["exp"])}
        data.update(self.Payload)
        return "[SignedToken %(jti)s sub=%(sub)s iss=%(iss)s exp=%(ctime)s]" % data

    @property
    def expiration(self):
        return self.Payload.get("exp")

    @property
    def issuer(self):
        return self.Payload.get("iss")

    @property
    def subject(self):
        return self.Payload.get("sub")

    @property
    def tid(self):
        return self.Payload.get("jti")

    @staticmethod   
    def from_bytes(encoded):
        encoded = to_bytes(encoded).strip()   # convert to bytes and remove all white space
        payload = jwt.decode(encoded, options={"verify_signature":False})
        token = SignedToken(payload=payload)
        token.Encoded = encoded
        return token

    decode = from_bytes
        
    def encode(self, key=None, private_key=None):
        if key is None and private_key is None:
            if self.Encoded:    return self.Encoded
            else:
                raise ValueError("Key is required")
        k = key or private_key
        encoded = jwt.encode(self.Payload, 
            key if private_key is None else private_key,
            algorithm="HS256" if private_key is None else "RS256"
        )
        self.Encoded = encoded
        return encoded
            
    def verify(self, key=None, public_key=None, ignore_times=False, leeway=0):

        if self.Encoded is None:
            raise ValueError("The ticket was not decoded")

        options = {}
        if ignore_times:
            options["verify_exp"] = False
            options["verify_nbf"] = False
        use_key = None
        algorithms = []
        if public_key:
            use_key = public_key
            algorithms=["RS256"]
        elif key:
            use_key = key
            algorithms=["HS256"]
        else:
            options["verify_signature"] = False
        
        self.Payload = jwt.decode(self.Encoded, use_key,
                leeway = timedelta(seconds=leeway),
                algorithms=algorithms,
                options = options
        )
    
    
    #
    # dict interface
    #
    
    def __getitem__(self, key):
        return self.Payload[key]
        
    def __setitem__(self, key, value):
        self.Payload[key] = value
        
    def get(self, key, default=None):
        return self.Payload.get(key, default)
        
    def keys(self):
        return self.Payload.keys()
    
    def items(self):
        return self.Payload.items()
    
    claims = items

        
if __name__ == "__main__":
    import os, pprint
    key = "abcd"

    payload = {"text":"hello world", "temp":32.0}
    print("Payload:")
    pprint.pprint(payload)
    
    t0 = SignedToken(payload, expiration=1000)
    print("Initial token:", t0)
    
    encoded = t0.encode(key=key)

    print("----------------")
    
    print("Encoded token:", encoded)
    
    print("----------------")
    
    
    t1 = SignedToken(encoded)
    t1.verify(key=key)
    print("Decoded token:",t1.Payload, t1.TID)
        
    
        



from .password_hash import password_hash, PasswordHashAlgorithm, password_digest_hash
from .rfc2617 import digest_client, digest_server
from .dbuser import BaseDBUser
from .signed_token_jwt import SignedToken, SignedTokenExpiredError, SignedTokenImmatureError, \
        SignedTokenUnacceptedAlgorithmError, SignedTokenSignatureVerificationError
from .token_box import TokenBox
from .token_lib import TokenLib
from .authenticators import authenticator
from .auth_client import TokenAuthClientMixin, AuthenticationError




import yaml

class MetaCatConfig(object):

    def __init__(self, cfg_file):
        if isinstance(cfg_file, str):
            cfg_file = open(cfg_file, "r")
            
        self.Config = yaml.load(cfg_file, Loader=yaml.SafeLoader)
    
    def __getitem__(self, name):
        return self.Config[name]



import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), "r").read()

def get_version():
    g = {}
    exec(open(os.path.join("metacat", "version.py"), "r").read(), g)
    return g["Version"]


setup(
    name = "metacat",
    version = get_version(),
    author = "Igor Mandrichenko",
    author_email = "ivm@fnal.gov",
    description = ("MetaCat is a general purpose metadata database. This package is the client side portion of the product."),
    license = "BSD 3-clause",
    keywords = "metadata, data management, database, web service",
    url = "https://github.com/ivmfnal/metacat",
    packages=['metacat', 'metacat.db', 'metacat.util', 'metacat.webapi', 'metacat.ui', 'metacat.auth', 'metacat.ui.cli'],
    install_requires=["pyjwt", "requests"],
    zip_safe = False,
    classifiers=[
    ],
    entry_points = {
            "console_scripts": [
                "metacat = metacat.ui.metacat_ui:main",
            ]
        }
)


import sys, getopt, os, json, fnmatch, pprint, datetime
#from urllib.request import urlopen, Request
from metacat.util import to_bytes, to_str, epoch
from metacat.webapi import MetaCatClient

from metacat.ui.cli import CLI, CLICommand

Usage = """
Usage: 
    metacat dataset <command> [<options>] ...
    
    Commands and options:
    
        list [<options>] [[<namespace pattern>:]<name pattern>]
            -l|--long           - detailed output
            -c|--file-counts    - include file counts if detailed output
            
        create [<options>] <namespace>:<name> [<description>]
            -M|--monotonic
            -F|--frozen
            -m|--metadata '<JSON expression>'
            -m|--metadata @<JSON file>
            
        add <parent dataset namespace>:<parent name> <child dataset namespace>:<child name> [<child dataset namespace>:<child name> ...]

        remove <parent namespace>:<parent name> <child namespace>:<child name> 
        
        show [<options>] <namespace>:<name>
            -j|--json       - print as JSON
            -p|--pprint     - Python pprint
        
        update <options> <namespace>:<name> [<description>]
            -M|--monotonic (yes|no) - set/reset monotonic flag
            -F|--frozen (yes|no)    - set/reset monotonic flag
            -r|--replace            - replace metadata, otherwise update
            -m|--metadata @<JSON file with metadata> 
            -m|--metadata '<JSON expression>' 
"""

class ListDatasetFilesCommand(CLICommand):
    
    Opts = "mj with-metadata"
    Usage = """[<options>] <dataset namespace>:<dataset name>           -- list dataset files
        -m|--with-metadata      - include file metadata
        -j                      - as JSON
    """
    MinArgs = 1
    
    def __call__(self, command, client, opts, args):
        dataset_did = args[0]
        with_meta = "-m" in opts or "--with-metadata" in opts
        files = client.get_dataset_files(dataset_did, with_metadata = with_meta)
        if "-j" in opts:
            first = True
            print("[")
            for f in files:
                if not first:   print(",")
                print(json.dumps(f, indent=2, sort_keys=True), end="")
                first = False
            print("\n]")
        else:
            for f in files:
                print(f"%s:%s" % (f["namespace"], f["name"]))

class ListDatasetsCommand(CLICommand):
    
    Opts = ("lc", ["--long", "--file-counts"])
    Usage = """ [<options>] [[<namespace pattern>:]<name pattern>]      -- list datasets
            -l|--long           - detailed output
            -c|--file-counts    - include file counts if detailed output
            """
    
    def __call__(self, command, client, opts, args):
        if args:
            patterns = args
        else:
            patterns = ["*"]
        opts = dict(opts)
        verbose = "-l" in opts or "--long" in opts
        include_counts = verbose and ("-c" in opts or "--file-counts" in opts)
        output = client.list_datasets(with_file_counts=include_counts)
    
        verbose_format = "%-16s %-19s %4d/%-4d %10s %s"
        header_format = "%-16s %-19s %9s %-10s %s"
    
        if verbose:
            print(header_format % (
                "creator", "created", "prnt/chld", "files", "namespace/name"
            ))
            print("-"*16, "-"*19, "-"*9, "-"*10, "-"*40)
    
        for item in output:
            match = False
            namespace, name = item["namespace"], item["name"]
            for p in patterns:
                pns = None
                if ":" in p:
                    pns, pn = p.split(":", 1)
                else:
                    pn = p
                if fnmatch.fnmatch(name, pn) and (pns is None or fnmatch.fnmatch(namespace, pns)):
                    match = True
                    break
            if match:
                if verbose:
                    nparents = len(item.get("parents", []))
                    nchildren = len(item.get("children", []))
                    ct = item.get("created_timestamp")
                    if not ct:
                        ct = ""
                    else:
                        ct = datetime.datetime.fromtimestamp(ct).strftime("%Y-%m-%d %H:%M:%S")
                    file_count = item.get("file_count")
                    if file_count is None:
                        file_count = ""
                    else:
                        file_count = str(file_count)
                    print(verbose_format % (
                        item.get("creator") or "",
                        ct,
                        nparents, nchildren,
                        file_count, 
                        namespace + ":" + name
                    ))
                else:
                    print("%s:%s" % (namespace, name))
                    

class ShowDatasetCommand(CLICommand):
    
    Opts = ("pj", ["pprint=","json"])
    Usage = """[<options>] <namespace>:<name>                           -- print dataset info
            -j|--json       - print as JSON
            -p|--pprint     - Python pprint
    """
    MinArgs = 1

    def __call__(self, command, client, opts, args):
        info = client.get_dataset(args[0])
        if "-p" in opts or "--pprint" in opts:
            pprint.pprint(info)
        elif "-j" in opts or "--json" in opts:
            print(json.dumps(info, indent=4, sort_keys=True))
        else:
            for k, v in sorted(info.items()):
                if k == "created_timestamp":
                    v = "" if not v else datetime.datetime.fromtimestamp(v).strftime("%Y-%m-%d %H:%M:%S")
                elif k == "children" or k == "parents":
                    n = len(v)
                    if n <= 5:
                        v = " ".join(v)
                    else:
                        v = "(%d) " % (n,) + " ".join(v[:5]) + " ..."
                elif k == "metadata":
                    v = json.dumps(v or {})
                print("%-25s: %s" % (k, v))
                    

class AddSubsetCommand(CLICommand):

    Usage = """<parent dataset namespace>:<parent name> <child dataset namespace>:<child name> [<child dataset namespace>:<child name> ...]
    """
    MinArgs = 2

    def __call__(self, command, client, opts, args):
        parent, children = args[0], args[1:]
        for child in children:
            client.add_child_dataset(parent, child)

def load_metadata(opts):
    # return None if no -j in options
    meta = None
    if "-m" in opts or "--metadata" in opts:
        arg = opts.get("-m") or opts.get("--metadata")
        if arg.startswith('@'):
            meta = json.load(open(arg[1:], "r"))
        else:
            meta = json.loads(arg)
    return meta
    
class CreateDatasetCommand(CLICommand):

    Opts = ("MFm:", ["monotonic", "frozen", "metadata="])
    Usage = """[<options>] <namespace>:<name> [<description>]           -- create dataset
        -M|--monotonic
        -F|--frozen
        -m|--metadata '<JSON expression>'
        -m|--metadata @<JSON file>
        """
    MinArgs = 1
        
    def __call__(self, command, client, opts, args):
        dataset_spec, desc = args[0], args[1:]
        if desc:
            desc = " ".join(desc)
        else:
            desc = ""    
        monotonic = "-M" in opts or "--monotonic" in opts
        frozen = "-F" in opts or "--frozen" in opts
        metadata = load_metadata(opts) or {}
        out = client.create_dataset(dataset_spec, monotonic = monotonic, frozen = frozen, description=desc, metadata = metadata)
        print(out)

class UpdateDatasetCommand(CLICommand):

    Opts = ("MFm:r", ["replace", "monotonic", "frozen", "metadata="])
    Usage = """<options> <namespace>:<name> [<description>]             -- modify dataset info
            -M|--monotonic (yes|no) - set/reset monotonic flag
            -F|--frozen (yes|no)    - set/reset monotonic flag
            -r|--replace            - replace metadata, otherwise update
            -m|--metadata @<JSON file with metadata> 
            -m|--metadata '<JSON expression>' 
    """
    MinArgs = 1

    def __call__(self, command, client, opts, args):
        mode = "replace" if ("-r" in opts or "--replace" in opts) else "update"

        if not args or args[0] == "help":
            print(Usage)
            sys.exit(2)
        
        metadata = load_metadata(opts)

        dataset = args[0]
        monotonic = frozen = None
        if "-M" in opts or "--monotonic" in opts:    
            monotonic = opts.get("-M") or opts.get("--monotonic")
            if not monotonic in ("yes", "no"):
                print("Invalid value for -M or --monotonic option:", monotonic, ". Valid values are 'yes' and 'no'")
                sys.exit(2)
            monotonic = monotonic == "yes"
        if "-F" in opts or "--frozen" in opts:    
            frozen = opts.get("-F") or opts.get("--frozen")
            if not frozen in ("yes", "no"):
                print("Invalid value for -F or --frozen option:", frozen, ". Valid values are 'yes' and 'no'")
                sys.exit(2)
            frozen = frozen == "yes"
        desc = None
        if args[1:]:
            desc = " ".join(args[1:])

        response = client.update_dataset(dataset, metadata=metadata, frozen=frozen, monotonic=monotonic, mode=mode, description=desc)
        print(response)

DatasetCLI = CLI(
    "create",       CreateDatasetCommand(),
    "show",         ShowDatasetCommand(),
    "files",        ListDatasetFilesCommand(),
    "list",         ListDatasetsCommand(),
    "add",          AddSubsetCommand(),
    "update",       UpdateDatasetCommand()
)
    
 
